#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use strict;
use Net::SMTP;
use Socket;
use POSIX qw(locale_h);
use locale;
use Locale::gettext;
use Mail::Internet;
use Vhffs::Main;
use Vhffs::Services::Mailing;
use Vhffs::Listengine::Intl;


#Huho, if the program stop heres, you have some problems with your MTA configuration
if( $#ARGV != 2 )
{
	print "involve as: listengine action local_part domain\n";
	exit 0;
}

my $action = shift;
my $lpart  = shift;
my $domain = shift;
my $vhffs = init Vhffs::Main;

my $DOMAIN		= $vhffs->get_config->get_listengine_domain;
my $LISTMASTER	= $vhffs->get_config->get_listengine_listmaster;
my $ADMIN		= $vhffs->get_config->get_listengine_listmaster;
my $SMTP_SERVER = $vhffs->get_config->get_listengine_smtp;

#Be careful, listengine will create /data/listengine/archives for archives
#and /data/listengine/tomoderate for moderation
my $DIRECTORY = $vhffs->get_config->get_listengine_datadir;


sub add_list_header
{
    my $header = shift;
    my $list = shift;

    $header->replace( 'List-Unsubscribe' ,"<mailto:".$list->get_localpart."-request"."\@".$list->get_domain."?subject=unsubscribe>" );
    $header->replace( 'List-Subscribe' ,"<mailto:".$list->get_localpart."-request"."\@".$list->get_domain."?subject=subscribe>" );
    $header->replace( 'List-Help' ,"<mailto:".$list->get_localpart."-request"."\@".$list->get_domain."?subject=help>" );
    $header->replace( 'List-Software' ,"Listengine 4.0, VHFFS 4.0" );
    $header->replace( 'List-Id' , $list->get_localpart.".".$list->get_domain );
}



sub sendmail
{
    my $mail  = shift;
    my $addrs = shift;
    my @tos;
    my $line;
    my $adr;
    my $smtp;
    my $header_pass = 0;


    if( ref($addrs) eq "ARRAY" )
    {
	my $v;
	foreach $v ( @{$addrs} )
	{
	    push @tos , $v;
	}
    }
    else
    {
	push @tos , $addrs;
   }

	foreach $adr (@tos)
	{
		$smtp = Net::SMTP->new( $SMTP_SERVER,
			Hello => $DOMAIN,
		);

		$smtp->mail( $mail->get('From') );
		$smtp->to( $adr );
		$smtp->data( $mail->as_string );
		$smtp->quit;
	}
}



sub archive_it
{
    my $mail = shift;
    my $list = shift;

    my ( $message_id  )=  ($mail->get( 'Message-Id' ) =~ /<(.+)>/);

    #Don't archive if message-id is not found
    return if( ( ! defined $message_id ) || ( $message_id eq "" ) );

    my $directory = $DIRECTORY . "/archives/" . $list->get_domain . "/" . $list->get_localpart;

    my ( undef, undef, undef, $day, $month, $year ) = gmtime(time);
    $directory .= sprintf ('/%.4u/%.2u/%.2u', $year+1900, $month+1, $day);

    Vhffs::Functions::create_dir( $directory ) if( ! -d $directory );

    my $file = $directory . "/" . $message_id;
    open( FILE , ">$file");
    print FILE $mail->as_string;
    close( FILE );
}


sub put_in_moderation
{
    my $mail = shift;
    my $list = shift;
    my @tos;
    my $members = $list->get_members;
    my $header;
    my $email;
    my ( $message_id  )=  ($mail->get( 'Message-Id' ) =~ /<(.+)>/);
    my ( $from ) = ( $mail->get('From') =~ /(([\+\.a-zA-Z_\-0-9^\s]+)@([\.a-zA-Z_\-0-9^\s]+)\.(\w+))/);
    my $directory = get_moderation_dir( $list );
    my $subject = $mail->get('Subject');

    Vhffs::Functions::create_dir( $directory ) if( ! -d $directory );
    
    my $filehash = Digest::MD5::md5_hex( $message_id );
    my $file = "$directory/".$filehash;
    
    
    
    open( FILE , ">$file");
    print FILE $mail->as_string;
    close( FILE );
    
    foreach ( keys %{$members} )
    {
	#Prepare the destinataire for the mail
	push( @tos , $_ ) if( $members->{$_}{perm} == Vhffs::Constants::ML_RIGHT_ADMIN );
    }

    $header = new Mail::Header( );
    $header->replace( 'From' ,  $LISTMASTER );
    $header->replace( 'Subject' ,  "moderate $filehash" );
    $header->replace( 'Reply-To' , $list->get_localpart ."-request"."\@".$list->get_domain);
    
    
    $email = Mail::Internet->new(  [ <> ]  , 
				   Header => $header,
				   Body => Vhffs::Listengine::Intl::moderate_message( $list , $filehash , $from , $subject )
				   );
    
    sendmail( $email , \@tos ) if( ( defined $email ) && ( $#tos >= 0 ) );
}



sub subscribe_to_list
{
    my $list = shift;
    my $from = shift;
    my $hash;
    my $header;
    my $email;
    my $message;
    my @body;

    if( $list->get_open_sub != 1 )
    {
	$message ="Subscribtion are not allowed for this list";
	push @body , $message ;	
	push @body , "\n";
	
	$header = new Mail::Header( );
	$header->replace( 'To' ,  $from );
	$header->replace( 'From' ,  $LISTMASTER );
	$header->replace( 'Subject' ,  "subscription denied" );
	
	
	$email = Mail::Internet->new(  [ <> ] , 
				       Header => $header ,
				       Body => Vhffs::Listengine::Intl::sub_deny( $list , $from )
				       );
    }
    else
    {
	
	if( defined ( $hash = $list->add_sub_with_reply( $from ) ) )
	{
	    $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $header->replace( 'Subject' ,  "confirm subscribe $hash" );
	    $header->replace( 'Reply-To' , $list->get_localpart ."-request"."\@".$list->get_domain);
	    
	    
	    $email = Mail::Internet->new(  [ <> ]  , 
					   Header => $header,
					   Body => Vhffs::Listengine::Intl::new_sub( $list , $from , $hash )
					   );
	    
	}
	else
	{
		    
	    $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $header->replace( 'Subject' ,  "already subscriber for the list" );
	    
	    
	    $email = Mail::Internet->new(  [ <> ]  , 
					   Header => $header,
					   Body => Vhffs::Listengine::Intl::sub_already_exist( $list , $from )
					   );

	}
    }

   
    sendmail( $email , $from ) if( defined $email );

}



sub unsubscribe_to_list
{
    my $list = shift;
    my $from = shift;
    my $hash;
    my $email;

    if( defined ( $hash = $list->del_sub_with_reply( $from ) ) )
    {
	
	my $header = new Mail::Header( );

	$header->replace( 'To' ,  $from );
	$header->replace( 'From' ,  $LISTMASTER );



	if( ( $hash == -1 ) || ( $hash == -2 ) )
	{
	    $header->replace( 'Subject' ,  "error while unsubscribe" );
	    $email = Mail::Internet->new(  [ <> ] , 
					   Header => $header ,
					   Body => Vhffs::Listengine::Intl::sub_not_exist( $list , $from )
					   );
	}
	elsif( $hash == -3 )
	{
	    $header->replace( 'Subject' ,  "Error while unsubscribe" );
	    #If $hash == -3, the subscriber was not a subscriber or admin but has different rights (waiting for confirm ...)
	    $email = Mail::Internet->new(  [ <> ] , 
					      Header => $header ,
					      Body => Vhffs::Listengine::Intl::error_unsub( $list , $from , $hash )
					      );
	}
	else
	{
	    $header->replace( 'Subject' ,  "confirm unsubscribe $hash" );
	    $header->replace( 'Reply-To' , $list->get_localpart ."-request"."\@".$list->get_domain);
	    
	    $email = Mail::Internet->new(  [ <> ] , 
					   Header => $header ,
					   Body => Vhffs::Listengine::Intl::new_unsub( $list , $from , $hash )
					   );
	}
	sendmail( $email , $from );
    }
}


sub confirm_sub
{
    my $list = shift;
    my $from = shift;
    my $hash = shift;


    my $subs = $list->get_members;
  

    #Check if su$bscription exists and the subscriber is waiting for reply
    if( ( defined $subs->{$from} ) && ( $subs->{$from}{'perm'} == Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_REPLY ) )
    {
	# Ok, hash are the same, treat the mail now
	if( $hash eq $subs->{$from}{'hash'} )
	{
	    my $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $header->replace( 'Subject' , "Successfully subscribe" );
    	    $header->replace('Date' , gmtime()." +00");
	    
	    my $email = Mail::Internet->new(  [ <> ] , 
					      Header => $header ,
					      Body => Vhffs::Listengine::Intl::confirm_sub( $list , $from )
					      );
	    sendmail( $email , $from );
	    
	    $list->change_state_for_sub( $from , Vhffs::Constants::ML_RIGHT_SUB );
	    $list->clear_hash( $from );
	}
	else
	{
	    my $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $header->replace( 'Subject' , "Subscribing error" );
    	    $header->replace('Date' , gmtime()." +00");
	    
	    my $email = Mail::Internet->new(  [ <> ] , 
					      Header => $header ,
					      Body => Vhffs::Listengine::Intl::error_sub( $list , $from )
					      );
	    sendmail( $email , $from );
	    
	}
    }

}

sub confirm_unsub
{
    my $list = shift;
    my $from = shift;
    my $hash = shift;
    my $email;
    my $header;
    my $subs = $list->get_members;
    
    #Check if subscription exists
    if( ( defined $subs->{$from} ) && ( $subs->{$from}{'perm'} == Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_DEL ) )
    {
	# Ok, hash are the same, treat the mail now and delete the subscriber from the list
	if( $hash eq $subs->{$from}{'hash'} )
	{
	    
	    $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $header->replace( 'Subject' , "Successfully unsubscribe" );
    	    $header->replace('Date' , gmtime()." +00");
	    
	    
	    $email = Mail::Internet->new(  [ <> ] , 
					   Header => $header ,
					   Body => Vhffs::Listengine::Intl::unsub_success( $list )
					   );
	    sendmail( $email , $from );
	    
	    $list->del_sub( $from );
	    
	}
	else
	{
	    #Here, hash are not the same, so, we return an error
	    $header = new Mail::Header( );
	    $header->replace( 'To' ,  $from );
	    $header->replace( 'From' ,  $LISTMASTER );
	    $email = Mail::Internet->new(  [ <> ] , 
					   Header => $header ,
					   Body => Vhffs::Listengine::Intl::error_unsub_hash( $list )
					   );
	}
	
    }   
}





#Change some mail headers and send it to each subscriber
sub sendmail_to_list
{
	my $mail = shift;
	my $list = shift;
	my $listmail = $list->get_localpart . "\@" . $list->get_domain;
	my $subs = $list->get_members;
   
	my @tos;

	#Replace the Reply-To field if selected in the panel  
	add_list_header( $mail , $list );
	$mail->replace( 'Reply-To' , $list->get_localpart ."\@" . $list->get_domain ) if( $list->get_replyto == 1 );


	#Add prefix if not null
	my $subject = $mail->get('Subject');
	my $tsubject = $subject;
	$tsubject =~ s/[\n\r]//g;
	my $prefix = $list->get_prefix;
	my $qprefix = quotemeta $prefix;
	unless ( $tsubject =~ /\[$qprefix\]/)
	{
		$mail->replace( 'Subject' , "[".$prefix."]"." ".$subject ) if( length( $prefix ) > 0 );
	}

	foreach ( keys %{$subs} )
	{
		#Send mail to user if he is a confirmed subscriber
		push( @tos , $_ ) if( $subs->{$_}{perm} != Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_REPLY  &&  $subs->{$_}{member} ne $listmail );
	}

	sendmail( $mail , \@tos );
	archive_it( $mail , $list );
}


sub bounce_mail
{

    my $mail = shift;
    my $list = shift;

    exit -1 if( ! defined $list );
    exit 0 if( $list->fetch < 0 );
    
    my ( $from ) = ( $mail->get('From') =~ /(([\+\.a-zA-Z_\-0-9^\s]+)@([a\.-zA-Z_\-0-9^\s]+)\.(\w+))/);
    my $subject = $mail->get('Subject');
    my $subs = $list->get_members;
    
    my $authorized = 0;
    my $in_moderation = 0;
    

    if( $list->get_moderated == 1 )
    {
       
    	$in_moderation = 1;

	# If the list is moderated, we refuse the mail
	# But if the mail is from an admin, we allow it
	foreach ( keys %{$subs} )
	{
	    if( ( $from eq $_ ) && ( $subs->{$_}{perm} == Vhffs::Constants::ML_RIGHT_ADMIN ) )
	    {
		$authorized = 1;
		$in_moderation = 0;
		last;
	    }
	}

    }
    elsif( $list->get_open_post == 1 )
    {
        # If the list is not moderated, and open_post,
        # we authorize the mail
	$authorized = 1;
    }
    else
    {
	# In this case, The list is not in open_post
	# We must check if the poster is a subscriber
	foreach ( keys %{$subs} )
	{
	    $authorized = 1 if( ( $from eq $_ ) && ( $subs->{$_}{perm} != Vhffs::Constants::ML_RIGHT_SUB_WAITING_FOR_REPLY ) );
	}
    }

    
    sendmail_to_list( $mail , $list ) if( $authorized == 1 );
    put_in_moderation( $mail , $list ) if( $in_moderation == 1 );
}


sub get_moderation_dir
{
    my $list = shift;
    return( $DIRECTORY . "/moderation/" . $list->get_domain . "/" . $list->get_localpart  );
}

sub validate_message
{
    my $list = shift;
    my $hash = shift;
    my @tempmail;
    chomp( $hash );
    my $dir  = get_moderation_dir( $list );
    my $file = $dir . "/" . $hash;
   
    return -1 if( ! -d $dir );
    return -2 if( ! -f $file );

    open( MODERATION , $file ) or return -2;

    while( <MODERATION> )
    {
	push( @tempmail , $_ );
    }
    close( MODERATION );


    return -3 if( unlink( $file ) <= 0 );

    my $m = Mail::Internet->new( \@tempmail );

    sendmail_to_list( $m , $list );
    
    return 1;
}


sub refuse_message
{
    my $list = shift;
    my $hash = shift;
    chomp( $hash );
    my $dir  = get_moderation_dir( $list );
    my $file = $dir . "/" . $hash;
    return -1 if( ! -d $dir );
    return -2 if( ! -f $file );


    return -1 if( unlink( $file ) == 0 );

    
    return 1;
}


sub list_msg_to_moderate{
    my $list = shift;
    my $dir = get_moderation_dir( $list );
    my @result;
    my @files;
    my $file;
    my $complete;
    my $mail;

    if( ! -d $dir )
    {
	push( @result , gettext("No message to moderate") );
	push( @result , "\n" );
    }
    else
    {
	opendir( DIR , $dir );
	@files = readdir( DIR );

	foreach $file ( @files )
	{
	    next if( ( $file eq '.' ) || ( $file eq '..' ) );
	    $complete = $dir . "/" . $file ;
	    
	    if( -f $complete )
	    {
		$mail = fetch_mail_from_file( $complete );
		next if( ! defined $mail );
		
		push( @result , "Sender: " . $mail->get('From:') );
		push( @result , "Subject: " . $mail->get('Subject:') );
		push( @result , "Id in listengine: " . $file );
		push( @result , "\n\n");
	    }
	}

	closedir( DIR );
    }
    
    return( \@result );
    
}



#################
# Treat a mail that seems ok
#
# This function treat request, sended to list-request@domain.tld
# It does not treat posts on list, sended to list@domain.tld
#################################
sub treat_request
{
    my $mail = shift;
    my $list = shift;

    my ( $from ) = ( $mail->get('From') =~ /(([\+\.a-zA-Z_\-0-9^\s]+)@([\.a-zA-Z_\-0-9^\s]+)\.(\w+))/);
    my $subject = $mail->get('Subject');
    my $action;
    my $hash;
    my $email;
    my $header;
    my $lang;
    my $temp;
    my $members = $list->get_members;

	$lang = Vhffs::Services::Mailing::get_lang_for_sub( $vhffs , $from );
	#If the user specified a lang, we change it to get internationalized messages
	if( defined $lang )
	{
		bindtextdomain("vhffs", $vhffs->get_config->get_intldir);
		textdomain("vhffs");
	}



    if( $subject =~ /^help$/ )
    {
		$email = Mail::Internet->new( [ <> ],
			     Body => Vhffs::Listengine::Intl::generate_help
			     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine help") );
    	$email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
    }
    elsif( $subject =~ /^subscribe$/ )
    {
		subscribe_to_list( $list , $from );
    }
    elsif( $subject =~ /^unsubscribe$/ )
    {
		unsubscribe_to_list( $list , $from );
    }
    elsif( ( $lang ) = $subject =~ /^lang\s([a-zA-Z\_\-]+)$/ )
    {
		#Try to change the language for this subscriber on whole listengine subsystem
		if( Vhffs::Services::Mailing::set_lang_for_sub( $vhffs , $from , $lang ) < 0 )
		{
		    $email = Mail::Internet->new( [ <> ],
					 Body => Vhffs::Listengine::Intl::lang_change_error( $from , $lang )
					 );
		    $email->replace('From' ,  $LISTMASTER );
		    $email->replace('To' ,  $from );
		    $email->replace('Subject' , gettext("listengine help") );
		    $email->replace('Date' , gmtime()." +00");
		    sendmail( $email , $from );
	}
	else
	{
	    $email = Mail::Internet->new( [ <> ],
					 Body => Vhffs::Listengine::Intl::lang_change_success( $from , $lang )
					 );
	    $email->replace('From' ,  $LISTMASTER );
	    $email->replace('To' ,  $from );
	    $email->replace('Subject' , gettext("listengine help") );
    	    $email->replace('Date' , gmtime()." +00");
	    sendmail( $email , $from );
	}
    }
    elsif( ( $temp ) = $subject =~ /^[.\s]*moderate\s([a-zA-Z0-9\s]+)$/ )
    {
	#To moderate messages, users must be at least ADMIN
	#So, we check this NOW
	if( $members->{$from}{perm} != Vhffs::Constants::ML_RIGHT_ADMIN )
	{
	    $email = Mail::Internet->new( [ <> ],
					 Body => Vhffs::Listengine::Intl::not_allowed( $list , $from )

					 );
	    $email->replace('From' ,  $LISTMASTER );
	    $email->replace('To' ,  $from );
	    $email->replace('Subject' , gettext("listengine result command") );
    	    $email->replace('Date' , gmtime()." +00");
	    sendmail( $email , $from );
	   
	}
	elsif( ( $hash ) = $temp =~ /validate\s([a-zA-Z0-9]+)$/ )
	{

	    if( validate_message( $list , $hash , $from ) < 0 )
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::moderate_error( $list , $hash )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
	    }
	    else
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::moderate_success( $list , $hash )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
	    }
	 
	}
	elsif( ( $hash ) = $temp =~ /refuse\s([a-zA-Z0-9]+)$/ )
	{
	    #Here, we refuse the message
	    if( refuse_message( $list , $hash , $from ) < 0 )
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::refuse_error( $list , $hash )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
		return;
	    }
	    else
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::refuse_success( $list , $hash )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
	    }
	    return;
	}
	elsif( $temp =~ /^list$/ )
	{
	    $email = Mail::Internet->new( [ <> ],
					 Body => list_msg_to_moderate( $list )
					 );
	    $email->replace('From' ,  $LISTMASTER );
	    $email->replace('To' ,  $from );
	    $email->replace('Subject' , sprintf( gettext("listengine - list of messages to moderate for %s") , $list->get_listname  ) );
    	    $email->replace('Date' , gmtime()." +00");
	    sendmail( $email , $from );
	    return;
	}
	elsif( $temp =~ /^[a-zA-Z0-9]+$/ )
	{
	    if( validate_message( $list , $temp , $from ) < 0 )
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::moderate_error( $list , $temp )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
	    }
	    else
	    {
		$email = Mail::Internet->new( [ <> ],
					     Body => Vhffs::Listengine::Intl::moderate_success( $list , $temp )
					     );
		$email->replace('From' ,  $LISTMASTER );
		$email->replace('To' ,  $from );
		$email->replace('Subject' , gettext("listengine moderation") );
    	        $email->replace('Date' , gmtime()." +00");
		sendmail( $email , $from );
	    }
	}
    }
    elsif( ( ( $action , $hash ) = $subject =~ /[.\s]*confirm\s([a-zA-Z0-9]+)\s(.+)$/i ) )
    {
		if( $action =~ /^subscribe$/i )
		{
		    #Here, this is a confirmation a subscribing
		    confirm_sub( $list , $from , $hash );
		}
		elsif( $action =~ /^unsubscribe$/i )
		{
		    #It's a confirmation to unsubscribe
		    confirm_unsub( $list , $from , $hash );
		}
       
    }
    else
    {
		$mail = Mail::Internet->new( [ <> ],
			     Body => Vhffs::Listengine::Intl::unknown_command( $list )
			     );
		$mail->replace('From' ,  $LISTMASTER );
		$mail->replace('To' ,  $from );
		$mail->replace('Subject' , gettext("listengine: unknown command") );
  	  	$mail->replace('Date' , gmtime()." +00");
		sendmail( $mail , $from );
    }

}


####################################
# Fetch the mail from standard input
#
# In fact, exim and others mail-server software give the mail
# through a pipe. So, the mail can be read by the standard input
###############
sub fetch_mail_from_stdin
{    
    my @mail;
    my $passed;
    my $line;
    $passed = 0;
    while( $line = <STDIN> )
    {
	if( $passed == 0 )
	{
	    if( $line =~ /^[a-zA-Z0-9\-]+:\s.+$/ )
	    {
		$passed = 1;
		push( @mail , $line );
	    }
	}
	else
	{
	    push( @mail , $line );
	}
    }
    my $m = Mail::Internet->new( \@mail );
    return $m;
}

sub fetch_mail_from_file
{    
    my @mail;
    my $passed;
    my $line;
    $passed = 0;

    my $file = shift;

    return -1 if( ! -f $file );
    
    open( FILE , $file ) or return -2;

    while( $line = <FILE> )
    {
	if( $passed == 0 )
	{
	    if( $line =~ /^[a-zA-Z0-9\-]+:\s.+$/ )
	    {
		$passed = 1;
		push( @mail , $line );
	    }
	}
	else
	{
	    push( @mail , $line );
	}
    }
   # my @mail = <STDIN>;
    my $m = Mail::Internet->new( \@mail );

    close( FILE ) or return -3;
    return $m;
}




#######################################
#Get language parameter for poster
#
#Internationalisation inside :-)
######################
sub get_lang
{
    my $mail = shift;
    my ( $from ) = ( $mail->get('From') =~ /(([\+\.a-zA-Z_\-0-9^\s]+)@([\.a-zA-Z_\-0-9^\s]+)\.(\w+))/);

    my $lang = Vhffs::Services::Mailing::get_lang_for_sub( $vhffs , $from );
    if( defined $lang )
    {	
	#Change current locale to user preferences if defined
		bindtextdomain("vhffs", $vhffs->get_config->get_intldir);
		textdomain("vhffs");
		setlocale(LC_ALL, $lang  );
    }
}


#####################################
# Verify if the mail is conform
#
##############################
sub verify_mail_with_list
{
        my $list = shift;
        my $mail = shift;

        my $from = $mail->get('From');

        my $temp = $list->get_localpart."-request\@".$list->get_domain;
        exit( 0 ) if( $from eq $list->get_listname);
        exit( 0 ) if( $from eq $temp);
}





###############################
# Ok, the program begins here
###############################


#Fetch the mail
my $mail = fetch_mail_from_stdin();

#Get language for subscriber and change locale if settings exists
get_lang( $mail );

#Build the list object from VHFFS
my $list = new Vhffs::Services::Mailing( $vhffs , $lpart , $domain );
exit( 0 ) if( $list->fetch < 0 );


verify_mail_with_list( $list , $mail );


####
# See what action to do with the list
if( $action eq 'bounce' )
{
    bounce_mail( $mail , $list  );
}
elsif( $action eq 'request' )
{
    treat_request( $mail , $list );
}
exit( 0 );
